' Module: XportXML
' Version: 1.0
' Date Written: 2009-04-09
' Author: Ashish Kumar
'
' Purpose: Creates export and import code for XML data from/to sql server (Tested with 2005 & 2008)
' License to use: Feel free to use this script for non commercial use.

' How it works:
' Needs specification file name as argument.
' File needs the following entries:
' 1st Line: Connection information of the source system
' 2nd Line: Connection information for the target system (You can change it later)
' 3rd Line: Export/Import options
' 4th line onwards: Each line must contain one entry per table.
' The entries should be comma separated and optionally delimited by double quotes.
' Refer to the accompanied documentation for details.

' Output:
' Script will create three files.
' 1. export_data.bat - Run this batch file to create output XML data.
' 2. import_data.vbs
' 3. exported_data.xsd - XML schema, which will be used by the import script

' Dependencies: SQL DMO, SQL Bulk Load to run the import.

' Notes:
' 2009-04-09: New code started

Option Explicit

Const ForReading = 1
Const ForWriting = 2
Const ForAppending = 8
Const TristateUseDefault = -2
Const TristateTrue = -1
Const TristateFalse = 0

Const xsConnStrTemplateSQLAuth = "Provider=SQLOLEDB.1;Password=%PWD%;User ID=%UID%;Initial Catalog=%DATABASE%;Data Source=%SERVER%"
Const xsConnStrTemplateWindowsAuth = "Provider=SQLOLEDB.1;Integrated Security=SSPI;Initial Catalog=%DATABASE%;Data Source=%SERVER%"

Const constMaxElements = 14
Const xsCurrLevel = 0
Const xsParentArrayIndex = 1
Const xsSrcTableOwner = 2
Const xsSrcTable = 3
Const xsTgtTableOwner = 4
Const xsTgtTable = 5
Const xsPkColArray = 6
Const xsFKColArray = 7
Const xsSQL = 8
Const xsHasChildren = 9
Const xsXSDFragment = 10
Const xsWhereClause = 11
Const xsSrcTableAlias = 12
Const xsCreateDummyColumn = 13
' SQL DMO Constants
Const SQLDMOKey_Foreign = 3

Dim myCRLF
Dim myIndent

Sub BuildScripts(strInFile)
    Dim oFS
    Dim oFile
    Dim oStream
    Dim strRec
    Dim strConnstrSrc
    Dim strSrcUID
    Dim strSrcPWD
    Dim strSrcServer
    Dim strSrcDatabase
    Dim bSrcWindowsAuthentication
    Dim strConnstrTgt
    Dim strTgtUID
    Dim strTgtPWD
    Dim strTgtServer
    Dim strTgtDatabase
    Dim bTgtWindowsAuthentication
    Dim nLineCnt
    Dim aTbls()
    Dim aTblx()
    Dim aTbl
    Dim aFK
    Dim nCnt1
    Dim nCnt2
    Dim nCurrLevel
    Dim strTempA
    Dim strTempB
    Dim strTempC
    Dim strSrcOwner
    Dim strSrcTable
    Dim strTgtOwner
    Dim strTgtTable
    Dim strSQL
    Dim myAlias
    Dim strXMLOutputFile
    Dim strXSDFile
    Dim strWhereClause
    Dim bDrop
    Dim bCreateSchema
    Dim bKeepIdentity
    Dim strFileLocation

    Set oFS = CreateObject("Scripting.FileSystemObject")
    Set oFile = oFS.GetFile(strInFile)
    Set oStream = oFile.OpenAsTextStream(ForReading, TristateUseDefault)
    Erase aTbls
    nLineCnt = 0
    nCnt1 = 0
    Do While Not oStream.AtEndOfStream
        strRec = Trim(oStream.ReadLine)
        If (Left(strRec, 1) = ";") Or (nLineCnt >= 3 And strRec="") Then
        Else
            nLineCnt = nLineCnt + 1
            Select Case nLineCnt
                Case 1
                    strSrcUID = parse_string(strRec, 1, ",", """")
                    strSrcPWD = parse_string(strRec, 2, ",", """")
                    strSrcServer = parse_string(strRec, 3, ",", """")
                    strSrcDatabase = parse_string(strRec, 4, ",", """")
                    bSrcWindowsAuthentication = parse_string(strRec, 5, ",", """")
                Case 2
                    strTgtUID = parse_string(strRec, 1, ",", """")
                    strTgtPWD = parse_string(strRec, 2, ",", """")
                    strTgtServer = parse_string(strRec, 3, ",", """")
                    strTgtDatabase = parse_string(strRec, 4, ",", """")
                    bTgtWindowsAuthentication = parse_string(strRec, 5, ",", """")
                Case 3
                    strFileLocation = parse_string(strRec, 1, ",", """")
                    bDrop = parse_string(strRec, 2, ",", """")
                    bCreateSchema = parse_string(strRec, 3, ",", """")
                    bKeepIdentity = parse_string(strRec, 4, ",", """")
                    If strFileLocation = "" Then
                        strFileLocation = "c:\"
                    Else
                        If Right(strFileLocation, 1) <> "\" Then
                            strFileLocation = strFileLocation & "\"
                        End If
                    End If
                    If bDrop = "Y" Then
                    Else
                        bDrop = "N" 'Default
                    End If
                    If bCreateSchema = "N" Then
                    Else
                        bCreateSchema = "Y" 'Default
                    End If
                    If bKeepIdentity = "Y" Then
                    Else
                        bKeepIdentity = "N" ' Default
                    End If
                Case Else
                    nCurrLevel = Left(strRec, 1)
                    strTempA = Trim(Mid(strRec, 2))
                    strTempB = Trim(parse_string(strTempA, 1, ",", """"))
                    strTempC = Trim(parse_string(strTempA, 2, ",", """"))
                    strWhereClause = Trim(parse_string(strTempA, 3, ",", """"))
                    If strWhereClause = "" Then
                        strWhereClause = "1=1"
                    End If
                    If parse_string(strTempB, 2, ".", """") = "" Then
                        strSrcOwner = "dbo"
                        strSrcTable = scrubTableName(parse_string(strTempB, 1, ".", """"))
                    Else
                        strSrcOwner = scrubTableName(parse_string(strTempB, 1, ".", """"))
                        strSrcTable = scrubTableName(parse_string(strTempB, 2, ".", """"))
                    End If
                    strSrcTable = GetCorrectedTableName( _
                                            strSrcUID, _
                                            strSrcPWD, _
                                            strSrcServer, _
                                            strSrcDatabase, _
                                            bSrcWindowsAuthentication, _
                                            strSrcOwner, _
                                            strSrcTable)
                    ' Build target table name
                    If strTempC = "" Then
                        strTgtOwner = "dbo"
                        strTgtTable = "_xs" & strSrcTable
                        If Len(strTgtTable) > 32 Then
                            ' Tf target table name exceeds then truncate it to 32 charactes
                            strTgtTable = Left(strTgtTable, 32)
                        End If
                    Else
                        If parse_string(strTempC, 2, ".", """") = "" Then
                            strTgtOwner = "dbo"
                            strTgtTable = scrubTableName(parse_string(strTempC, 1, ".", """"))
                        Else
                            strTgtOwner = scrubTableName(parse_string(strTempC, 1, ".", """"))
                            strTgtTable = scrubTableName(parse_string(strTempC, 2, ".", """"))
                        End If
                    End If
                    nCnt1 = nCnt1 + 1
                    ReDim Preserve aTbls(nCnt1 - 1)
                    Erase aTblx
                    ReDim aTblx(constMaxElements - 1)
                    aTbl = aTblx
                    aTbl(xsCurrLevel) = nCurrLevel
                    aTbl(xsParentArrayIndex) = 0  ' For now set the parent's index to 0.
                    aTbl(xsSrcTableOwner) = strSrcOwner
                    aTbl(xsSrcTable) = strSrcTable
                    aTbl(xsTgtTableOwner) = strTgtOwner
                    aTbl(xsTgtTable) = strTgtTable
                    aTbl(xsPkColArray) = GetPKList( _
                                            strSrcUID, _
                                            strSrcPWD, _
                                            strSrcServer, _
                                            strSrcDatabase, _
                                            bSrcWindowsAuthentication, _
                                            strSrcOwner, _
                                            strSrcTable)
                    If nCurrLevel = 1 Then
                        aTbl(xsHasChildren) = "Y"
                    Else
                        aTbl(xsHasChildren) = "N"
                    End If
                    aTbl(xsWhereClause) = strWhereClause
                    aTbl(xsSrcTableAlias) = "" & MakeConformedName(strSrcTable)
                    aTbl(xsCreateDummyColumn) = 0
                    aTbls(nCnt1 - 1) = aTbl
            End Select
        End If
    Loop
    oStream.Close
    ' Let us populate parent's indicies
    For nCnt1 = GetUBound(aTbls) To 0 Step -1
        aTbl = aTbls(nCnt1)
        If aTbl(xsCurrLevel) = 1 Then
            ' Nothing to do
        Else
            For nCnt2 = nCnt1 - 1 To 0 Step -1
                If aTbls(nCnt2)(xsCurrLevel) < aTbl(xsCurrLevel) Then
                    aTbl(xsParentArrayIndex) = nCnt2
                    aTbl(xsFKColArray) = GetFKList( _
                                            strSrcUID, _
                                            strSrcPWD, _
                                            strSrcServer, _
                                            strSrcDatabase, _
                                            bSrcWindowsAuthentication, _
                                            "" & aTbls(nCnt1)(xsSrcTableOwner), _
                                            "" & aTbls(nCnt1)(xsSrcTable), _
                                            "" & aTbls(nCnt2)(xsSrcTableOwner), _
                                            "" & aTbls(nCnt2)(xsSrcTable))
                    aTbls(nCnt1) = aTbl
                    ' Set the HasChildren flag on
                    aTbls(nCnt2)(xsHasChildren) = "Y"
                    aTbl = aTbls(nCnt1)(xsFKColArray)
                    If GetUBound(aTbl) = -1 Then
                        aTbls(nCnt1)(xsCreateDummyColumn) = 1
                        aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsCreateDummyColumn) = 1
                    End If
                    Exit For
                End If
            Next
        End If
    Next
'   Let's rock and roll
    For nCnt1 = 0 To GetUBound(aTbls)
        myAlias = "" & aTbls(nCnt1)(xsSrcTableAlias)
        strSQL = "select "
        strSQL = strSQL & myCRLF & GetColList( _
                                        strSrcUID, _
                                        strSrcPWD, _
                                        strSrcServer, _
                                        strSrcDatabase, _
                                        bSrcWindowsAuthentication, _
                                        "" & aTbls(nCnt1)(xsSrcTableOwner), _
                                        "" & aTbls(nCnt1)(xsSrcTable), _
                                        myAlias) & _
                 "," & _
                 myCRLF & _
                 "<SUBQUERY>"
        strSQL = strSQL & _
                 myCRLF & _
                 "from " & _
                 myCRLF & _
                 myIndent & _
                 UnscrubTableName(strSrcDatabase) & _
                 "." & _
                 UnscrubTableName("" & aTbls(nCnt1)(xsSrcTableOwner)) & _
                 "." & _
                 UnscrubTableName("" & aTbls(nCnt1)(xsSrcTable)) & _
                 " " & _
                 myAlias
        strSQL = strSQL & _
                 myCRLF & _
                 "where " & _
                 myCRLF & _
                 aTbls(nCnt1)(xsWhereClause)
        If aTbls(nCnt1)(xsCurrLevel) > 1 Then
            ' Build the joins now
            aFK = aTbls(nCnt1)(xsFKColArray)
            For nCnt2 = 0 To GetUBound(aFK)
                strSQL = strSQL & _
                         " and "
                strSQL = strSQL & _
                         myCRLF & _
                         myIndent
                strSQL = strSQL & _
                         myAlias & _
                         "." & _
                         aTbls(nCnt1)(xsFKColArray)(nCnt2) & _
                         " = " & _
                         aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsSrcTableAlias) & _
                         "." & _
                         aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsPkColArray)(nCnt2)
            Next
        End If
        strSQL = strSQL & _
                 myCRLF & _
                 "FOR XML AUTO, ELEMENTS XSINIL, TYPE, ROOT ('grp" & aTbls(nCnt1)(xsSrcTableAlias) & "')"
        aTbls(nCnt1)(xsSQL) = strSQL
        'Debug.Print strSQL
    Next
    For nCnt1 = GetUBound(aTbls) To 0 Step -1
        If aTbls(nCnt1)(xsCurrLevel) = 1 Then
            ' Do nothing
        Else
            strSQL = aTbls(nCnt1)(xsSQL)
            strSQL = "(" & Replace(strSQL, "," & myCRLF & "<SUBQUERY>", "") & ")"
            strTempA = aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsSQL)
            strTempA = Replace(strTempA, "<SUBQUERY>", strSQL & "," & myCRLF & "<SUBQUERY>")
            aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsSQL) = strTempA
        End If
    Next
    strSQL = aTbls(0)(xsSQL)
    strSQL = Replace(strSQL, "," & myCRLF & "<SUBQUERY>", "")
    'Debug.Print strSQL
    ' Build the script to extract data
    'strXMLOutputFile = strFileLocation & "exported_data_" & Format(Now, "YYYYMMDDHHNNSS") & ".xml"
    strXMLOutputFile = strFileLocation & "exported_data.xml"
    strXSDFile = Replace(strXMLOutputFile, ".xml", ".xsd")
    BuildBCPExport _
                strSrcUID, _
                strSrcPWD, _
                strSrcServer, _
                strSrcDatabase, _
                bSrcWindowsAuthentication, _
                strSQL, _
                strXMLOutputFile, _
                strFileLocation & "export_data.bat"

    BuildXSDSchema _
                strSrcUID, _
                strSrcPWD, _
                strSrcServer, _
                strSrcDatabase, _
                bSrcWindowsAuthentication, _
                aTbls, _
                strXSDFile

    BuildImportScript _
                strTgtUID, _
                strTgtPWD, _
                strTgtServer, _
                strTgtDatabase, _
                bSrcWindowsAuthentication, _
                strXMLOutputFile, _
                strXSDFile, _
                strFileLocation & "import_data.vbs", _
                aTbls, _
                bDrop, _
                bCreateSchema, _
                bKeepIdentity
                
    MsgBox "Script creation done!" & vbCrLf & _
           "Scripts location: " & strFileLocation & vbCrLf & _
           "Export batch file: export_data.bat" & vbCrLf & _
           "Import script: import_data.vbs" & vbCrLf & _
           "XSD Schema: exported_data.xsd"
    Exit Sub
BuildScripts_Err:
    MsgBox Err.Description & "(" & Err.Number & ")"
End Sub

Function GetCorrectedTableName( _
        strUID, _
        strPWD, _
        strServer, _
        strDatabase, _
        bWindowsAuth, _
        strOwner, _
        strTableName)
    ' This function returns the table name in original case (upper/lower)
    Dim oDMO 'As New SQLDMO.SQLServer
    Dim oDB 'As New SQLDMO.Database
    Dim oTbl 'As New SQLDMO.Table
    Dim oCol 'As SQLDMO.Column
    
    Set oDMO = CreateObject("SQLDMO.SQLServer")
    Set oDB = CreateObject("SQLDMO.Database")
    Set oTbl = CreateObject("SQLDMO.Table")
    Set oCol = CreateObject("SQLDMO.Column")
    
    If bWindowsAuth = "Y" Then
        oDMO.LoginSecure = True
        oDMO.Connect strServer
    Else
        oDMO.LoginSecure = False
        oDMO.Connect strServer, strUID, strPWD
    End If
    Set oDB = oDMO.Databases(strDatabase)
    Set oTbl = oDB.Tables(strOwner & "." & strTableName)
    GetCorrectedTableName = oTbl.Name
End Function

Function GetColList( _
        strUID, _
        strPWD, _
        strServer, _
        strDatabase, _
        bWindowsAuth, _
        strOwner, _
        strTableName, _
        strAlias)
        
    Dim nCnt1
    Dim nCnt2
    Dim strTemp
    Dim oDMO 'As New SQLDMO.SQLServer
    Dim oDB 'As New SQLDMO.Database
    Dim oTbl 'As New SQLDMO.Table
    Dim oCol 'As SQLDMO.Column
    
    Set oDMO = CreateObject("SQLDMO.SQLServer")
    Set oDB = CreateObject("SQLDMO.Database")
    Set oTbl = CreateObject("SQLDMO.Table")
    Set oCol = CreateObject("SQLDMO.Column")
    
    If bWindowsAuth = "Y" Then
        oDMO.LoginSecure = True
        oDMO.Connect strServer
    Else
        oDMO.LoginSecure = False
        oDMO.Connect strServer, strUID, strPWD
    End If
    Set oDB = oDMO.Databases(strDatabase)
    strTemp = ""
    Set oTbl = oDB.Tables(strOwner & "." & strTableName)
    For nCnt2 = 1 To oTbl.Columns.Count
        If nCnt2 > 1 Then
            strTemp = strTemp & "," & myCRLF
        End If
        If oTbl.Columns(nCnt2).DataType = "datetime" Or _
            oTbl.Columns(nCnt2).DataType = "smalldatetime" Then
            strTemp = strTemp & _
                      myIndent & _
                      "convert(varchar," & _
                      strAlias & "." & _
                      oTbl.Columns(nCnt2).Name & "," & _
                      "121) as " & _
                      oTbl.Columns(nCnt2).Name
        Else
            strTemp = strTemp & myIndent & strAlias & "." & oTbl.Columns(nCnt2).Name
        End If
    Next
    GetColList = strTemp
End Function

Function GetPKList( _
        strUID, _
        strPWD, _
        strServer, _
        strDatabase, _
        bWindowsAuth, _
        strOwner, _
        strTableName)
    Dim aPK()
    Dim nCnt1
    Dim nCnt2
    
    Dim oDMO 'As New SQLDMO.SQLServer
    Dim oDB 'As New SQLDMO.Database
    Dim oTbl 'As New SQLDMO.Table
    Dim oCol 'As SQLDMO.Column
    Dim oKey 'As New SQLDMO.Key
    
    Set oDMO = CreateObject("SQLDMO.SQLServer")
    Set oDB = CreateObject("SQLDMO.Database")
    Set oTbl = CreateObject("SQLDMO.Table")
    Set oCol = CreateObject("SQLDMO.Column")
    Set oKey = CreateObject("SQLDMO.Key")

    If bWindowsAuth = "Y" Then
        oDMO.LoginSecure = True
        oDMO.Connect strServer
    Else
        oDMO.LoginSecure = False
        oDMO.Connect strServer, strUID, strPWD
    End If
    Set oDB = oDMO.Databases(strDatabase)
    Erase aPK
    Set oTbl = oDB.Tables(strOwner & "." & strTableName)
    If oTbl.PrimaryKey Is Nothing Then
    Else
        Set oKey = oTbl.PrimaryKey
        For nCnt2 = 1 To oKey.KeyColumns.Count
            ReDim Preserve aPK(nCnt2 - 1)
            aPK(nCnt2 - 1) = oKey.KeyColumns(nCnt2)
        Next
    End If
    GetPKList = aPK
End Function

Function GetFKList( _
            strUID, _
            strPWD, _
            strServer, _
            strDatabase, _
            bWindowsAuth, _
            strOwner, _
            strTableName, _
            strParentOwner, _
            strParentTable)
    Dim aFK()
    Dim nCnt1
    Dim nCnt2
    Dim nCnt3
    
    Dim oDMO 'As New SQLDMO.SQLServer
    Dim oDB 'As New SQLDMO.Database
    Dim oTbl 'As New SQLDMO.Table
    Dim oCol 'As SQLDMO.Column
    Dim oKey 'As New SQLDMO.Key
    
    Set oDMO = CreateObject("SQLDMO.SQLServer")
    Set oDB = CreateObject("SQLDMO.Database")
    Set oTbl = CreateObject("SQLDMO.Table")
    Set oCol = CreateObject("SQLDMO.Column")
    Set oKey = CreateObject("SQLDMO.Key")
    
    If bWindowsAuth = "Y" Then
        oDMO.LoginSecure = True
        oDMO.Connect strServer
    Else
        oDMO.LoginSecure = False
        oDMO.Connect strServer, strUID, strPWD
    End If
    Set oDB = oDMO.Databases(strDatabase)
    Erase aFK
    Set oTbl = oDB.Tables(strOwner & "." & strTableName)
    ' Loop through key collection to find FKs
    For nCnt2 = 1 To oTbl.Keys.Count
        If oTbl.Keys(nCnt2).Type = SQLDMOKey_Foreign And _
            scrubTableName(oTbl.Keys(nCnt2).ReferencedTable) = strOwner & "." & strParentTable Then
            For nCnt3 = 1 To oTbl.Keys(nCnt2).KeyColumns.Count
                ReDim Preserve aFK(nCnt3 - 1)
                aFK(nCnt3 - 1) = oTbl.Keys(nCnt2).KeyColumns(nCnt3)
            Next
            Exit For
        End If
    Next
    GetFKList = aFK
End Function

Function scrubTableName(strTableName)
    Dim strTemp
    strTemp = Replace(strTableName, "[", "")
    strTemp = Replace(strTemp, "]", "")
    scrubTableName = strTemp
End Function

Function UnscrubTableName(strTableName)
    Dim strTemp
    strTemp = "[" & strTableName & "]"
    UnscrubTableName = strTemp
End Function

Function MakeConformedName(strTableName)
    Dim strTemp
    strTemp = scrubTableName(strTableName)
    strTemp = Replace(strTemp, " ", "_")
    MakeConformedName = strTemp
End Function

Sub BuildBCPExport( _
            strUID, _
            strPWD, _
            strServer, _
            strDatabase, _
            bWindowsAuth, _
            strSQL, _
            strOutputFile, _
            strScriptFileName)
            
    Dim oFS
    Dim oFile
    Dim oStream
    Dim strTempA
    
    Set oFS = CreateObject("Scripting.FileSystemObject")
    Set oFile = oFS.createTextFile(strScriptFileName, True)

    oFile.WriteLine "rem This file was created automatically on " & Now()
    oFile.WriteLine "rem Run this script to export XML data"
    oFile.WriteLine "echo Starting to export data"
    strTempA = "bcp"
    strTempA = strTempA & _
               " """ & Replace(Replace(strSQL, vbLf, " "), vbCr, "") & """"
    strTempA = strTempA & _
               " queryout"
    strTempA = strTempA & _
               " """ & strOutputFile & """"
    If bWindowsAuth = "Y" Then
        strTempA = strTempA & _
                   " -T"
    Else
        strTempA = strTempA & _
                   " -U" & strUID
        strTempA = strTempA & _
                   " -P" & strPWD
    End If
    strTempA = strTempA & _
               " -S" & strServer
    strTempA = strTempA & _
               " -c -r -t -C65001"
    'oFile.WriteLine """" & Replace(strTempA, """", """""") & """"
    oFile.WriteLine strTempA
    oFile.WriteLine "echo XML Data export completed"
    oFile.Close
End Sub

Sub BuildXSDSchema( _
    strUID, _
    strPWD, _
    strServer, _
    strDatabase, _
    bWindowsAuth, _
    aTbls(), _
    strXSDFile)
    
    Dim oFS
    Dim oFile
    Dim oStream
    Dim strConnStr
    Dim nCnt1
    Dim nCnt2
    Dim nCnt3
    Dim strTempA
    Dim strTempB
    Dim strParentOwner
    Dim strParentTable
    Dim nParentIdx
    Dim strOwner
    Dim strTable
    Dim strTgtOwner
    Dim strTgtTable
    Dim aFK

    Dim oDMO 'As New SQLDMO.SQLServer
    Dim oDB 'As New SQLDMO.Database
    Dim oTbl 'As New SQLDMO.Table
    Dim oCol 'As SQLDMO.Column
    
    Set oDMO = CreateObject("SQLDMO.SQLServer")
    Set oDB = CreateObject("SQLDMO.Database")
    Set oTbl = CreateObject("SQLDMO.Table")
    Set oCol = CreateObject("SQLDMO.Column")
    
    If bWindowsAuth = "Y" Then
        oDMO.LoginSecure = True
        oDMO.Connect strServer
    Else
        oDMO.LoginSecure = False
        oDMO.Connect strServer, strUID, strPWD
    End If
    Set oDB = oDMO.Databases(strDatabase)
    ' Update individual XSD Fragments
    For nCnt3 = 0 To GetUBound(aTbls)
        strTempA = ""
        strOwner = UnscrubTableName("" & aTbls(nCnt3)(xsSrcTableOwner))
        strTable = UnscrubTableName("" & aTbls(nCnt3)(xsSrcTable))
        strTgtOwner = UnscrubTableName("" & aTbls(nCnt3)(xsTgtTableOwner))
        strTgtTable = UnscrubTableName("" & aTbls(nCnt3)(xsTgtTable))
        'If aTbls(nCnt3)(xsHasChildren) = "Y" Then
            strTempA = strTempA & " <xs:element name=^Q^grp" & aTbls(nCnt3)(xsSrcTableAlias) & "^Q^ sql:is-constant=^Q^true^Q^>" & myCRLF
            strTempA = strTempA & "  <xs:complexType>" & myCRLF
            strTempA = strTempA & "   <xs:sequence>" & myCRLF
        'End If
        strTempA = strTempA & "    <xs:element name=^Q^" & aTbls(nCnt3)(xsSrcTableAlias) & "^Q^ sql:relation=^Q^" & strTgtOwner & "." & strTgtTable & "^Q^"
        If aTbls(nCnt3)(xsCurrLevel) > 1 Then
            strTempA = strTempA & " sql:relationship=^Q^R" & nCnt3 & "^Q^"
        End If
        strTempA = strTempA & ">" & myCRLF
        strTempA = strTempA & "     <xs:complexType>" & myCRLF
        strTempA = strTempA & "      <xs:sequence>" & myCRLF
'       Embed the column list
        Set oTbl = oDB.Tables(strOwner & "." & UnscrubTableName("" & aTbls(nCnt3)(xsSrcTable)))
        For nCnt2 = 1 To oTbl.Columns.Count
            strTempA = strTempA & _
                       "<xs:element name=^Q^" & _
                       oTbl.Columns(nCnt2).Name & _
                       "^Q^ sql:field=^Q^" & _
                       oTbl.Columns(nCnt2).Name & _
                       "^Q^ type=^Q^xs:" & _
                       GetXSDDatatype(oTbl.Columns(nCnt2).DataType) & _
                       "^Q^/>" & _
                       myCRLF
        Next
        If aTbls(nCnt3)(xsHasChildren) = "Y" Then
            strTempA = strTempA & "<xsdCHILDREN>"
        End If
        strTempA = strTempA & "      </xs:sequence>" & myCRLF
        strTempA = strTempA & "     </xs:complexType>" & myCRLF
        strTempA = strTempA & "    </xs:element>" & myCRLF
        'If aTbls(nCnt3)(xsHasChildren) = "Y" Then
            strTempA = strTempA & "   </xs:sequence>" & myCRLF
            strTempA = strTempA & "  </xs:complexType>" & myCRLF
            strTempA = strTempA & " </xs:element>" & myCRLF
        'End If
        aTbls(nCnt3)(xsXSDFragment) = strTempA
    Next
    For nCnt1 = GetUBound(aTbls) To 0 Step -1
        If aTbls(nCnt1)(xsCurrLevel) = 1 Then
            ' Do nothing
        Else
            strTempA = aTbls(nCnt1)(xsXSDFragment)
            strTempB = aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsXSDFragment)
            strTempB = Replace(strTempB, "<xsdCHILDREN>", strTempA & "<xsdCHILDREN>")
            aTbls(aTbls(nCnt1)(xsParentArrayIndex))(xsXSDFragment) = strTempB
        End If
    Next
    ' At the end of this iteration, the root node has all the children
    
    Set oFS = CreateObject("Scripting.FileSystemObject")
    Set oFile = oFS.createTextFile(strXSDFile, True)
    
    WriteQuotedLine oFile, "<?xml version=^Q^1.0^Q^ encoding=^Q^utf-8^Q^ ?>"
    WriteQuotedLine oFile, "<xs:schema xmlns:sql=^Q^urn:schemas-microsoft-com:mapping-schema^Q^"
    WriteQuotedLine oFile, "                  xmlns=^Q^http://tempuri.org/XMLSchema.xsd^Q^"
    WriteQuotedLine oFile, "                  xmlns:xs=^Q^http://www.w3.org/2001/XMLSchema^Q^>"
    WriteQuotedLine oFile, " <xs:annotation>"
    WriteQuotedLine oFile, " <xs:appinfo>"
    For nCnt3 = 1 To GetUBound(aTbls)
        nParentIdx = aTbls(nCnt3)(xsParentArrayIndex)
        strParentOwner = UnscrubTableName("" & aTbls(nParentIdx)(xsTgtTableOwner))
        strParentTable = UnscrubTableName("" & aTbls(nParentIdx)(xsTgtTable))
        strOwner = UnscrubTableName("" & aTbls(nCnt3)(xsTgtTableOwner))
        strTable = UnscrubTableName("" & aTbls(nCnt3)(xsTgtTable))
    
        WriteQuotedLine oFile, "  <sql:relationship name=^Q^R" & nCnt3 & "^Q^"
        WriteQuotedLine oFile, "     parent=^Q^" & strParentOwner & "." & strParentTable & "^Q^"
        WriteQuotedLine oFile, "     child=^Q^" & strOwner & "." & strTable & "^Q^"
        strTempA = ""
        strTempB = ""
        aFK = aTbls(nCnt3)(xsFKColArray)
        If GetUBound(aFK) > -1 Then
            For nCnt2 = 0 To GetUBound(aFK)
                If nCnt2 > 0 Then
                    strTempA = strTempA & ","
                    strTempB = strTempB & ","
                End If
                strTempA = strTempA & aTbls(nParentIdx)(xsPkColArray)(nCnt2)
                strTempB = strTempB & aTbls(nCnt3)(xsFKColArray)(nCnt2)
            Next
        Else
            ' Create dummy relationship.
            ' These dummy columns will get added to the table
            strTempA = "xsDummy"
            strTempB = "xsDummy"
        End If
        WriteQuotedLine oFile, "     parent-key=^Q^" & strTempA & "^Q^"
        WriteQuotedLine oFile, "     child-key=^Q^" & strTempB & "^Q^/>"
    Next
    
    WriteQuotedLine oFile, " </xs:appinfo>"
    WriteQuotedLine oFile, " </xs:annotation>"
    strTempB = aTbls(aTbls(0)(xsParentArrayIndex))(xsXSDFragment)
    strTempB = Replace(strTempB, "<xsdCHILDREN>", "")
    WriteQuoted oFile, Trim(strTempB)
    WriteQuotedLine oFile, "</xs:schema>"
    oFile.Close
End Sub

Sub WriteQuotedLine( _
        oFile, _
        strLine)
    oFile.WriteLine (Replace(strLine, "^Q^", """"))
End Sub

Sub WriteQuoted( _
        oFile, _
        strLine)
    oFile.Write (Replace(strLine, "^Q^", """"))
End Sub

Sub BuildImportScript( _
                strUID, _
                strPWD, _
                strServer, _
                strDatabase, _
                bWindowsAuth, _
                strXMLOutputFile, _
                strXSDFile, _
                strScriptFileName, _
                aTbls(), _
                bDrop, _
                bCreateSchema, _
                bKeepIdentity)

    Dim oFS
    Dim oFile
    Dim oStream
    Dim strTempA
    Dim strConnStr
    Dim nCnt1
    
    strConnStr = BuildConnectionString( _
                        strUID, _
                        strPWD, _
                        strServer, _
                        strDatabase, _
                        bWindowsAuth)
    
    Set oFS = CreateObject("Scripting.FileSystemObject")
    Set oFile = oFS.createTextFile(strScriptFileName, True)

    oFile.WriteLine "rem This file was created automatically on " & Now()
    oFile.WriteLine "rem Run this script to import XML data"
    oFile.WriteLine "Dim objBL"
    oFile.WriteLine "Dim objConn"
    
    oFile.WriteLine ""
    oFile.WriteLine "Set objConn = CreateObject(""ADODB.Connection"")"
    oFile.WriteLine "objConn.Open """ & strConnStr & """"
    oFile.WriteLine "on error resume next"
    For nCnt1 = 0 To GetUBound(aTbls)
        If aTbls(nCnt1)(xsCreateDummyColumn) <> 0 Then
            oFile.WriteLine "objConn.Execute ""alter table " & _
                            UnscrubTableName("" & aTbls(nCnt1)(xsTgtTableOwner)) & "." & _
                            UnscrubTableName("" & aTbls(nCnt1)(xsTgtTable)) & " " & _
                            "add xsDummy nvarchar(1)"""
        End If
    Next
    oFile.WriteLine "on error goto 0"
    oFile.WriteLine "Set objBL = CreateObject(""SQLXMLBulkLoad.SQLXMLBulkLoad.4.0"")"
    oFile.WriteLine "objBL.ConnectionString = """ & strConnStr & """"
    oFile.WriteLine "objBL.ErrorLogFile = ""error.xml"""
    oFile.WriteLine "objBL.KeepNulls = True"
    oFile.WriteLine "'objBL.Transaction = True"
    If bDrop = "Y" Then
        oFile.WriteLine "objBL.SGDropTables = True"
    Else
        oFile.WriteLine "objBL.SGDropTables = false"
    End If
    If bCreateSchema = "Y" Then
        oFile.WriteLine "objBL.SchemaGen = True"
    Else
        oFile.WriteLine "objBL.SchemaGen = False"
    End If
    If bKeepIdentity = "Y" Then
        oFile.WriteLine "objBL.KeepIdentity = True"
    Else
        oFile.WriteLine "objBL.KeepIdentity = False"
    End If
    oFile.WriteLine "objBL.Execute """ & strXSDFile & """, """ & strXMLOutputFile & """"
    oFile.WriteLine "Set objBL = Nothing"
    oFile.WriteLine "on error resume next"
    For nCnt1 = 0 To GetUBound(aTbls)
        If aTbls(nCnt1)(xsCreateDummyColumn) <> 0 Then
            oFile.WriteLine "objConn.Execute ""alter table " & _
                            UnscrubTableName("" & aTbls(nCnt1)(xsTgtTableOwner)) & "." & _
                            UnscrubTableName("" & aTbls(nCnt1)(xsTgtTable)) & " " & _
                            "drop column xsDummy"""
        End If
    Next
    oFile.WriteLine "on error goto 0"
    oFile.WriteLine "Wscript.Echo ""Import completed"""
    oFile.Close
End Sub

Function BuildConnectionString( _
                strUID, _
                strPWD, _
                strServer, _
                strDatabase, _
                bWindowsAuth)
    Dim strConnStr
    If bWindowsAuth = "Y" Then
        strConnStr = xsConnStrTemplateWindowsAuth
    Else
        strConnStr = xsConnStrTemplateSQLAuth
    End If
    strConnStr = Replace(strConnStr, "%UID%", strUID)
    strConnStr = Replace(strConnStr, "%PWD%", strPWD)
    strConnStr = Replace(strConnStr, "%DATABASE%", strDatabase)
    strConnStr = Replace(strConnStr, "%SERVER%", strServer)
    BuildConnectionString = strConnStr
End Function

Function GetXSDDatatype(strSQLDataType)
    Dim strTempA
    strTempA = "string"
    Select Case strSQLDataType
        Case "bigint"
            strTempA = "long"
        Case "binary"
            strTempA = "base64Binary"
        Case "bit"
            strTempA = "boolean"
        Case "char"
            strTempA = "string"
        Case "datetime"
            strTempA = "dateTime"
        Case "decimal"
            strTempA = "decimal"
        Case "float"
            strTempA = "double"
        Case "image"
            strTempA = "base64Binary"
        Case "int"
            strTempA = "int"
        Case "money"
            strTempA = "decimal"
        Case "nchar"
            strTempA = "string"
        Case "ntext"
            strTempA = "string"
        Case "nvarchar"
            strTempA = "string"
        Case "numeric"
            strTempA = "decimal"
        Case "real"
            strTempA = "float"
        Case "smalldatetime"
            strTempA = "dateTime"
        Case "smallint"
            strTempA = "short"
        Case "smallmoney"
            strTempA = "decimal"
        Case "sql_variant"
            strTempA = "string"
        Case "sysname"
            strTempA = "string"
        Case "text"
            strTempA = "string"
        Case "timestamp"
            strTempA = "dateTime"
        Case "tinyint"
            strTempA = "unsignedByte"
        Case "varbinary"
            strTempA = "base64Binary"
        Case "varchar"
            strTempA = "string"
        Case "uniqueidentifier"
            strTempA = "string"
    End Select
    GetXSDDatatype = Trim(strTempA)
End Function

Public Function parse_string(strParseMe, lngDelimPos, strDelim, strEncl)
' This function returns the nth(II arguement) parsed string supplied within parse_me$, delimited by
' supplied delimiter (III arguement)
' 10/10/2001 AK - Added the optionally enclosed argument.
    Dim nStopAtDelim, nCurrentDelimCnt, ptr, strRet, bTanking, nLen, nDelimLen
    Dim myEncl, bEnclosed
    
    myEncl = Left(strEncl, 1)
    strRet = ""
    If strDelim = "" Or lngDelimPos < 1 Then
        parse_string = strRet
        Exit Function
    End If
    bTanking = False
    nStopAtDelim = lngDelimPos - 1
    If nStopAtDelim = 0 Then
        bTanking = True
    End If
    nCurrentDelimCnt = 0
    nLen = Len(strParseMe)
    nDelimLen = Len(strDelim)
    ptr = 1
    bEnclosed = False
    If myEncl <> "" Then
        If Mid(strParseMe, 1, 1) = myEncl Then
            If bTanking Then
                bEnclosed = True
                ptr = 2
            End If
        End If
    End If
    Do While True
        Do While True
            If ptr > nLen Then
                parse_string = strRet
                Exit Function
            End If
            If myEncl <> "" Then
                If bTanking Then
                    ' I am looking for field enclosing delimiter
                    If Mid(strParseMe, ptr, 1) = myEncl Then
                        If bEnclosed Then
                            If Mid(strParseMe, ptr + 1, 1) = myEncl Then
                                ' Next is also a delimiter. Meaning delimiter as data
                                ptr = ptr + 1
                                ' Do not skip this iteration.  We need to tank this character
                            Else
                                bTanking = False
                                bEnclosed = False
                                ptr = ptr + 1
                                Exit Do
                            End If
                        End If
                    End If
                Else
                    If Mid(strParseMe, ptr, 1) = myEncl Then
                        If bEnclosed Then
                            If Mid(strParseMe, ptr + 1, 1) = myEncl Then
                                ' Next is also a delimiter. Meaning delimiter as data
                                ptr = ptr + 1
                            Else
                                bEnclosed = False
                                ptr = ptr + 1
                                Exit Do
                            End If
                        End If
                    End If
                End If
            End If
            If (Mid(strParseMe, ptr, nDelimLen) = strDelim) And Not bEnclosed Then
                nCurrentDelimCnt = nCurrentDelimCnt + 1
                If nCurrentDelimCnt = nStopAtDelim Then
                    bTanking = True
                    ptr = ptr + nDelimLen
                    If myEncl <> "" Then
                        If Mid(strParseMe, ptr, 1) = myEncl Then
                            bEnclosed = True
                            ptr = ptr + 1
                            Exit Do
                        End If
                    End If
                    Exit Do
                ElseIf nCurrentDelimCnt > nStopAtDelim Then
                    parse_string = strRet
                    Exit Function
                Else
                    ptr = ptr + nDelimLen
                    If myEncl <> "" Then
                        If Mid(strParseMe, ptr, 1) = myEncl Then
                            bEnclosed = True
                            ptr = ptr + 1
                            Exit Do
                        End If
                    End If
                    Exit Do
                End If
            End If
            If bTanking Then
                strRet = strRet & Mid(strParseMe, ptr, 1)
            End If
            ptr = ptr + 1
            Exit Do
        Loop
    Loop
    parse_string = strRet
End Function

Function GetUBound(aArr)
    GetUBound = -1
    On Error Resume Next
    GetUBound = UBound(aArr)
End Function

'Sub RunMe
    myCRLF = vbCrLf
    myIndent = vbTab
'   Main code starts here.
    BuildScripts WScript.Arguments.Item(0)
'End Sub